# -*- coding: utf-8 -*-
"""
  flatcms : A simple CMS for Django
  (C)2010 - Apidev - http://www.apidev.fr
  This is distributed under GNU LGPL license, see license.txt
"""

from django.contrib import admin
from django.utils.translation import ugettext_lazy as _
from django.forms.widgets import TextInput
from django.db.models import CharField
import models
import widgets
from django.utils.safestring import mark_safe
from django.forms import forms
from django.core.urlresolvers import reverse
from django.utils.encoding import smart_unicode
from django.conf import settings
        
class PageConfigWidget(widgets.SelectAndLinkWidget):
    model = models.PageConfig

class MetaHeaderInline(admin.TabularInline):
    """MetaHeader are displayed as a table into the SiteConfig admin page"""
    model = models.MetaHeader
    extra = 3

class SiteConfigAdmin(admin.ModelAdmin):
    """Customize SiteConfig admin page"""
    list_display = ('site', 'name', 'default_config', 'logo', 'favicon')
    fieldsets = (
        (_('General'), {'fields': ('site', 'name', 'default_config', 'custom_banner', 'show_title_in_content',
                                   'show_login_banner', 'show_site_in_title',)}),
        (_('Images'), {'fields': ('logo', 'favicon')}),
        (_('Copyright'), {'fields': ('designer', 'designer_website', 'design_year')}),
        (_('Google Analytics'), {'fields': ('analytics_option', 'analytics_code')}),
    )

    formfield_overrides = {
        CharField: {'widget': TextInput(attrs={'size':'100'})},
    }
    def formfield_for_dbfield(self, db_field, **kwargs):
        x = widgets.replace_widget(kwargs, db_field, ('logo', 'favicon'), widgets.AdminImageWidget)
        if not x:
            x = widgets.replace_widget(kwargs, db_field, ('default_config',), PageConfigWidget)
        return x if x else super(SiteConfigAdmin,self).formfield_for_dbfield(db_field,**kwargs)

    inlines = [MetaHeaderInline]
    save_on_top = True
    save_as = True

admin.site.register(models.SiteConfig, SiteConfigAdmin)

class ImageAdmin(admin.ModelAdmin):
    """Customize Image admin page"""
    list_display = ('display', 'name', 'image', 'image_size', 'file_size', 'ordering')
    fieldsets = (
        (_('General'), {'fields': ('name', 'image', 'image_size', 'file_size')}),
        (_('Image list'), {'fields': ('ordering', )}),
    )
    list_display_links = ('name', 'image',)
    list_editable = ('ordering', )
    date_hierarchy = 'lastmod'
    save_on_top = True
    save_as = True
    list_per_page = 20
    readonly_fields = ('image_size', 'file_size')
    search_fields = ('name', 'image')
    list_filter = ('image',)
    
    def display(self, obj):
        try:
            height = min(100, obj.image.height)
            if height==obj.image.height and obj.image.width>250:
                height = height/2
            return mark_safe(u'<img src="{0}" height="{1}px">'.format(obj.url(), height))
        except Exception:
            return mark_safe(_(u'{0}').format(obj.url()))
    display.allow_tags = True
    display.short_description = _(u'Image')
    
    def image_size(self, obj):
        try:
            return u'{0.width} x {0.height}'.format(obj.image)
        except Exception, msg:
            return u'{0}'.format(msg)
    image_size.short_description = _(u'Image size')
    
    def file_size(self, obj):
        try:
            if obj.image.size<1024:
                return _(u'{0} Bytes').format(obj.image.size)
            return _(u'{0} KB').format(obj.image.size/1024)
        except Exception, msg:
            return u'???'
    file_size.short_description = _('File size')
    
    def formfield_for_dbfield(self, db_field, **kwargs):
        x = widgets.replace_widget(kwargs, db_field, ('image',), widgets.AdminImageWidget)
        return x if x else super(ImageAdmin,self).formfield_for_dbfield(db_field,**kwargs)

admin.site.register(models.Image, ImageAdmin)

class SlideshowAdmin(admin.ModelAdmin):
    """Customize Image admin page"""
    list_display = ('name', 'width', 'height', 'effect', 'speed', 'timeout', 'random')
    fieldsets = (
        (_('General'), {'fields': ('name', 'images')}),
        (_('Options'), {'fields': ('width', 'height', 'effect', 'speed', 'timeout', 'random')}),
    )
    raw_id_fields = ('images',)
    save_on_top = True
    save_as = True

    def formfield_for_dbfield(self, db_field, **kwargs):
        x = widgets.replace_widget(kwargs, db_field,
            ('images',), widgets.ImageManyToManyRawIdWidget(db_field.rel))
        return x if x else super(SlideshowAdmin,self).formfield_for_dbfield(db_field,**kwargs)
        
admin.site.register(models.Slideshow, SlideshowAdmin)

class CssPropertyInline(admin.TabularInline):
    """CssEntry are displayed as a table into the CssSelector admin page"""
    model = models.CssProperty
    extra = 10

class CssSelectorAdmin(admin.ModelAdmin):
    """Customize CssSelector admin page"""
    list_display = ('selector', 'enabled', 'ordering', 'in_editor')
    fieldsets = (
        (_('General'), {'fields': ('site', 'selector', 'enabled', 'ordering', 'in_editor')}),
    )
    inlines = [CssPropertyInline]
    list_editable = ('enabled', 'ordering', 'in_editor')
    list_filter = ('site', 'enabled', 'in_editor','selector')
    search_fields = ['selector', 'cssproperty__property', 'cssproperty__value']
    save_on_top = True
    save_as = True

admin.site.register(models.CssSelector, CssSelectorAdmin)

def get_cross_reference(obj, obj_class, qs):
    edit_button_template = """
        <a onclick="return showAddAnotherPopup(this);" class="add-another"
        href="{0}">
        <img height="10" width="10" alt="{2}" title="{2}" src="{1}flatcms/img/icon_changelink.gif">
        </a>
    """
    
    html = u'<ul>'
    app_name, model_name = obj_class._meta.app_label, obj_class._meta.module_name
    for item in qs:
        edit_url = reverse('admin:{0}_{1}_change'.format(app_name,  model_name), args=[item.id])
        edit_button = edit_button_template.format(edit_url, settings.MEDIA_URL, _(u'Edit'))
        html +=  u'<li>{0}{1}</li>'.format(item, edit_button)
    return mark_safe(html+u'</ul>')

class PluginAdmin(admin.ModelAdmin):
    """Customize CmsPage admin page"""
    
    def _media(self):
        """Use Tiny_MCE as content editor"""
        return super(PluginAdmin, self).media + \
            forms.Media(js=('tiny_mce/tiny_mce.js', reverse('tinymce_config.js')))
    media = property(_media)
    
    def image_links(self, obj):
        return widgets.render_flatcms_image_links()
    image_links.short_description = _(u'Images')

    list_display = ('name', 'div_id', 'ordering', 'published', 'registration_required', 'py_code')
    fieldsets = (
        (_('General'), {'fields': ('name', 'div_id', 'ordering')}),
        (_('Content'), {'fields': ('image_links', 'py_code', 'content', 'slideshow')}),
        (_('Publication'), {'fields': ('published', 'registration_required')}),
        (_('Page configuration'), {'fields': ('used_by', 'not_used_by')}),
    )
    list_editable = ('published', 'registration_required', 'ordering')
    list_filter = ('published', 'registration_required', 'div_id', 'slideshow')
    readonly_fields = ('image_links', 'used_by', 'not_used_by')
    save_on_top = True
    save_as = True
    
    def used_by(self, obj):
        return get_cross_reference(obj, models.PageConfig, obj.pageconfig_set.all().order_by('id'))
    used_by.short_description = _(u'Configurations using this plugin')
    
    def not_used_by(self, obj):
        ids = [x.id for x in obj.pageconfig_set.all()]
        qs = models.PageConfig.objects.all().exclude(id__in=ids).order_by('id')
        return get_cross_reference(obj, models.PageConfig, qs)
    not_used_by.short_description = _(u'Configurations NOT using this plugin')

admin.site.register(models.Plugin, PluginAdmin)

class MenuItemInline(admin.TabularInline):
    """MenuItems are displayed as a table into the Menu admin page"""
    model = models.MenuItem
    fk_name = 'parent_menu'
    extra = 5

class MenuAdmin(admin.ModelAdmin):
    """Customize the Menu admin page"""
    list_display = ('label', 'div_id', 'ordering', 'published', 'registration_required')
    fieldsets = (
        (_('General'), {'fields': ('label', 'div_id', 'ordering')}),
        (_('Publication'), {'fields': ('published', 'registration_required')}),
        (_('Page configuration'), {'fields': ('used_by', 'not_used_by', 'used_by_menu')}),
    )
    list_editable = ('published', 'registration_required', 'ordering')
    list_filter = ('published', 'registration_required')
    readonly_fields = ('used_by', 'not_used_by', 'used_by_menu')
    inlines = [MenuItemInline]
    save_on_top = True
    save_as = True

    def used_by(self, obj):
        return get_cross_reference(obj, models.PageConfig, obj.pageconfig_set.all().order_by('id'))
    used_by.short_description = _(u'Configurations using this menu')
    
    def not_used_by(self, obj):
        ids = [x.id for x in obj.pageconfig_set.all()]
        qs = models.PageConfig.objects.all().exclude(id__in=ids).order_by('id')
        return get_cross_reference(obj, models.PageConfig, qs)
    not_used_by.short_description = _(u'Configurations NOT using this menu')
    
    def used_by_menu(self, obj):
        ids = [mi.parent_menu.id for mi in models.MenuItem.objects.filter(child_menu=obj)]
        qs = models.Menu.objects.filter(id__in=ids).order_by('id')
        return get_cross_reference(obj, models.Menu, qs)
    used_by_menu.short_description = _(u'Menus using this child menu')

admin.site.register(models.Menu, MenuAdmin)

class HeaderIncludeArgumentInline(admin.TabularInline):
    model = models.HeaderIncludeArgument
    extra = 5

class HeaderIncludeAdmin(admin.ModelAdmin):
    inlines = [HeaderIncludeArgumentInline]
    save_on_top = True
    save_as = True
    
admin.site.register(models.HeaderInclude, HeaderIncludeAdmin)

class PageOptionsInline(admin.TabularInline):
    model = models.PageOption
    extra = 2

class PageConfigAdmin(admin.ModelAdmin):
    """Customize the Menu admin page"""
    list_display = ('name', 'lang', 'template_name', 'step_banner')
    fieldsets = (
        (_('General'), {'fields': ('name', 'lang', 'template_name', 'header_includes', 'step_banner')}),
        (_('Items'), {'fields': ('menu', 'plugins')}),
    )
    list_editable = ('lang', 'template_name', 'step_banner')
    list_filter = ('plugins', 'menu', 'lang', 'template_name')
    filter_horizontal = ('plugins',)
    inlines = [PageOptionsInline]
    save_on_top = True
    save_as = True

admin.site.register(models.PageConfig, PageConfigAdmin)

class CmsPageAdmin(admin.ModelAdmin):
    """Customize CmsPage admin page"""
    def _media(self):
        """Use Tiny_MCE as content editor"""
        return super(CmsPageAdmin, self).media + \
            forms.Media(js=('tiny_mce/tiny_mce.js', reverse('tinymce_config.js'), 'flatcms/js/urlify_patch.js'))
    media = property(_media)

    def image_links(self, obj):
        return widgets.render_flatcms_image_links()
    image_links.short_description = _(u'Images')
    
    def lastmod_date(self, obj):
        return obj.lastmod.strftime('%d/%m/%Y %H:%M')
    lastmod_date.short_description = _(u'Last modification')

    list_display = ('url', 'site', 'title', 'published', 'registration_required', 'show_in_rss', 'lastmod_date', 'config',)
    list_display_links = ('url', 'title') 
    search_fields = ('url', )
    date_hierarchy = 'lastmod'
    list_editable = ('published', 'registration_required', 'show_in_rss','config')
    list_filter = ('site', 'config', 'url', 'published', 'registration_required', 'app_only', 'sitemap', 'show_in_rss')
    prepopulated_fields = {'url': ('title',)}
    readonly_fields = ('image_links',)
    save_on_top = True
    save_as = True
    fieldsets = (
        (_('General'), {'fields': ('site', 'title', 'url', 'description', 'config',)}),
        (_('Publication'), {'fields': ('published', 'registration_required',)}),
        (_('Content'), {'fields': ('image_links', 'content',)}),
        (_('Breadcrumbs'), {'fields': ('breadcrumb_text', 'breadcrumb_parent',)}),
        (_('Misc.'), {'fields': ('step_level', 'app_only',)}),
        (_('Sitemap'), {'fields': ('sitemap', 'priority', 'changefreq', 'show_in_rss',)}),
    )
    formfield_overrides = {
        CharField: {'widget': TextInput(attrs={'size':'100'})},
        
    }
    def formfield_for_dbfield(self, db_field, **kwargs):
        x = widgets.replace_widget(kwargs, db_field, ('config',), PageConfigWidget)
        return x if x else super(CmsPageAdmin,self).formfield_for_dbfield(db_field,**kwargs)
    
    
admin.site.register(models.CmsPage, CmsPageAdmin)

class AliasAdmin(admin.ModelAdmin):
    list_display = ('url', 'site', 'page', 'config', 'redirect', 'sitemap')
    list_display_links = ('url', 'page') 
    search_fields = ('url', 'page__title')
    list_filter = ('site', 'config', 'sitemap')
    list_editable = ('redirect', 'sitemap', 'config')
    save_on_top = True
    save_as = True
    fieldsets = (
        (_('Source'), {'fields': ('site', 'url')}),
        (_('Destination'), {'fields': ('page', 'config', 'redirect')}),
        (_('Sitemap'), {'fields': ('sitemap', )}),
    )
    formfield_overrides = {
        models.PageUrlField: {'widget': TextInput(attrs={'size':'100'})},
    }
admin.site.register(models.Alias, AliasAdmin)

class DownloadAdmin(admin.ModelAdmin):
    list_display = ('file', 'size')
    search_fields = ('file',)
    list_filter = ('file',)
    fieldsets = (
        (_('General'), {'fields': ('file', 'size')}),
    )
    readonly_fields = ('size',)
    save_on_top = True
    save_as = True
    
    def size(self, obj):
        try:
            if obj.file.size<1024:
                return _(u'{0} Bytes').format(obj.file.size)
            return _(u'{0} KB').format(obj.file.size/1024)
        except Exception:
            return '???'
    size.short_description = _('Size')
        
admin.site.register(models.Download, DownloadAdmin)

class StepInline(admin.TabularInline):
    """Steps are displayed as a table into the StepBanner admin page"""
    model = models.Step
    extra = 5

class StepBannerAdmin(admin.ModelAdmin):
    """Customize the StepBanner admin page"""
    list_display = ('name', 'show_num', 'show_all')
    fieldsets = (
        (_('General'), {'fields': ('name', 'show_num', 'show_all')}),
    )
    inlines = [StepInline]
    save_on_top = True
    save_as = True
admin.site.register(models.StepBanner, StepBannerAdmin)

admin.site.register(models.DjangoPage)